@@ -26,7 +26,7 @@
 #include "item.h"
 #include "relation.h"
 
-#include <QtCore/QObject>
+#include <QObject>
 
 namespace Akonadi
 {
@@ -38,10 +38,12 @@ class ItemFetchScope;
 class MonitorPrivate;
 class Session;
 class TagFetchScope;
+class NotificationSubscriber;
+class ChangeNotification;
 
 namespace Protocol
 {
-class ChangeNotification;
+class Command;
 }
 
 /**
@@ -91,7 +93,25 @@ public:
         Collections = 1,
         Items,
         Tags,
-        Relations
+        Relations,
+        /**
+         * Listen to subscription changes of other Monitors connected to Akonadi.
+         * This is only for debugging purposes and should not be used in real
+         * applications.
+         * @since 5.4
+         */
+        Subscribers,
+        /**
+         * Listens to all notifications being emitted by the server and provides
+         * additional information about them. This is only for debugging purposes
+         * and should not be used in real applications.
+         *
+         * @note Enabling monitoring this type has performance impact on the
+         * Akonadi Server.
+         *
+         * @since 5.4
+         */
+        Notifications
     };
 
     /**
@@ -99,12 +119,12 @@ public:
      *
      * @param parent The parent object.
      */
-    explicit Monitor(QObject *parent = Q_NULLPTR);
+    explicit Monitor(QObject *parent = nullptr);
 
     /**
      * Destroys the monitor.
      */
-    virtual ~Monitor();
+    ~Monitor() override;
 
     /**
      * Sets whether the specified collection shall be monitored for changes. If
@@ -194,7 +214,7 @@ public:
     void setAllMonitored(bool monitored = true);
 
     void setExclusive(bool exclusive = true);
-    bool exclusive() const;
+    Q_REQUIRED_RESULT bool exclusive() const;
 
     /**
      * Ignores all change notifications caused by the given session. This
@@ -253,7 +273,6 @@ public:
      *
      * @param enable @c true to enable the feature, @c false means everything
      * that was requested will be fetched.
-     * @return void
      */
     void fetchChangedOnly(bool enable);
 
@@ -328,7 +347,7 @@ public:
      *
      * @since 4.3
      */
-    Collection::List collectionsMonitored() const;
+    Q_REQUIRED_RESULT Collection::List collectionsMonitored() const;
 
     /**
      * Returns the set of items being monitored.
@@ -337,63 +356,63 @@ public:
      *
      * @since 4.6
      */
-    QVector<Item::Id> itemsMonitoredEx() const;
+    Q_REQUIRED_RESULT QVector<Item::Id> itemsMonitoredEx() const;
 
     /**
      * Returns the number of items being monitored.
      * Optimization.
      * @since 4.14.3
      */
-    int numItemsMonitored() const;
+    Q_REQUIRED_RESULT int numItemsMonitored() const;
 
     /**
      * Returns the set of mimetypes being monitored.
      *
      * @since 4.3
      */
-    QStringList mimeTypesMonitored() const;
+    Q_REQUIRED_RESULT QStringList mimeTypesMonitored() const;
 
     /**
      * Returns the number of mimetypes being monitored.
      * Optimization.
      * @since 4.14.3
      */
-    int numMimeTypesMonitored() const;
+    Q_REQUIRED_RESULT int numMimeTypesMonitored() const;
 
     /**
      * Returns the set of tags being monitored.
      *
      * @since 4.13
      */
-    QVector<Tag::Id> tagsMonitored() const;
+    Q_REQUIRED_RESULT QVector<Tag::Id> tagsMonitored() const;
 
     /**
      * Returns the set of types being monitored.
      *
      * @since 4.13
      */
-    QVector<Type> typesMonitored() const;
+    Q_REQUIRED_RESULT QVector<Type> typesMonitored() const;
 
     /**
      * Returns the set of identifiers for resources being monitored.
      *
      * @since 4.3
      */
-    QList<QByteArray> resourcesMonitored() const;
+    Q_REQUIRED_RESULT QList<QByteArray> resourcesMonitored() const;
 
     /**
      * Returns the number of resources being monitored.
      * Optimization.
      * @since 4.14.3
      */
-    int numResourcesMonitored() const;
+    Q_REQUIRED_RESULT int numResourcesMonitored() const;
 
     /**
      * Returns true if everything is being monitored.
      *
      * @since 4.3
      */
-    bool isAllMonitored() const;
+    Q_REQUIRED_RESULT bool isAllMonitored() const;
 
     /**
      * Sets the session used by the Monitor to communicate with the %Akonadi server.
@@ -408,7 +427,7 @@ public:
      *
      * @since 4.4
      */
-    Session *session() const;
+    Q_REQUIRED_RESULT Session *session() const;
 
     /**
      * Allows to enable/disable collection move translation. If enabled (the default), move
@@ -673,6 +692,57 @@ Q_SIGNALS:
      */
     void relationRemoved(const Akonadi::Relation &relation);
 
+    /**
+     * This signal is emitted when Subscribers are monitored and a new subscriber
+     * subscribers to the server.
+     *
+     * @param subscriber The new subscriber
+     * @since 5.4
+     *
+     * @note Monitoring for subscribers and listening to this signal only makes
+     * sense if you want to globally debug Monitors. There is no reason to use
+     * this in regular applications.
+     */
+    void notificationSubscriberAdded(const Akonadi::NotificationSubscriber &subscriber);
+
+    /**
+     * This signal is emitted when Subscribers are monitored and an existing
+     * subscriber changes its subscription.
+     *
+     * @param subscriber The changed subscriber
+     * @since 5.4
+     *
+     * @note Monitoring for subscribers and listening to this signal only makes
+     * sense if you want to globally debug Monitors. There is no reason to use
+     * this in regular applications.
+     */
+    void notificationSubscriberChanged(const Akonadi::NotificationSubscriber &subscriber);
+
+    /**
+     * This signal is emitted when Subscribers are monitored and an existing
+     * subscriber unsubscribes from the server.
+     *
+     * @param subscriber The removed subscriber
+     * @since 5.4
+     *
+     * @note Monitoring for subscribers and listening to this signal only makes
+     * sense if you want to globally debug Monitors. There is no reason to use
+     * this in regular applications.
+     */
+    void notificationSubscriberRemoved(const Akonadi::NotificationSubscriber &subscriber);
+
+    /**
+     * This signal is emitted when Notifications are monitored and the server emits
+     * anny change notification.
+     *
+     * @since 5.4
+     *
+     * @note Getting introspection into all change notifications only makes sense
+     * if you want to globally debug Notifications. There is no reason to use
+     * this in regular applications.
+     */
+    void debugNotification(const Akonadi::ChangeNotification &notification);
+
     /**
      * This signal is emitted if the Monitor starts or stops monitoring @p collection explicitly.
      * @param collection The collection
@@ -733,12 +803,14 @@ Q_SIGNALS:
      */
     void typeMonitored(const Akonadi::Monitor::Type type, bool monitored);
 
+    void monitorReady();
+
 protected:
     //@cond PRIVATE
     friend class EntityTreeModel;
     friend class EntityTreeModelPrivate;
     MonitorPrivate *d_ptr;
-    explicit Monitor(MonitorPrivate *d, QObject *parent = Q_NULLPTR);
+    explicit Monitor(MonitorPrivate *d, QObject *parent = nullptr);
     //@endcond
 
 private:
@@ -748,7 +820,8 @@ private:
     Q_PRIVATE_SLOT(d_ptr, void slotSessionDestroyed(QObject *))
     Q_PRIVATE_SLOT(d_ptr, void slotStatisticsChangedFinished(KJob *))
     Q_PRIVATE_SLOT(d_ptr, void slotFlushRecentlyChangedCollections())
-    Q_PRIVATE_SLOT(d_ptr, void slotNotify(const Akonadi::Protocol::ChangeNotification &))
+    Q_PRIVATE_SLOT(d_ptr, void slotUpdateSubscription())
+    Q_PRIVATE_SLOT(d_ptr, void handleCommands())
     Q_PRIVATE_SLOT(d_ptr, void dataAvailable())
     Q_PRIVATE_SLOT(d_ptr, void serverStateChanged(Akonadi::ServerManager::State))
     Q_PRIVATE_SLOT(d_ptr, void invalidateCollectionCache(qint64))
